<?php

/**
 * @file ReviewerCreditsEditorialDecisionReportPlugin.inc.php
 *
 * Copyright (c) 2014-2020 Simon Fraser University
 * Copyright (c) 2003-2020 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @class ReviewReportPlugin
 * @ingroup plugins_reports_review
 * @see ReviewReportDAO
 *
 * @brief Review report plugin
 */

import( 'lib.pkp.classes.plugins.ReportPlugin' );

class ReviewerCreditsEditorialDecisionReportPlugin extends ReportPlugin {
	/**
	 * ReviewerCreditsEditorialDecisionReportPlugin constructor.
	 */
	public function __construct() {

		if ( Config::getVar( 'general', 'installed' ) ) {
			require_once( 'plugins/generic/reviewerCredits/ReviewerCreditsEditorialDecisionReportDAO.inc.php' );
			$editorialDecisionReportDAO = new ReviewerCreditsEditorialDecisionReportDAO();
			DAORegistry::registerDAO( 'ReviewerCreditsEditorialDecisionReportDAO', $editorialDecisionReportDAO );
		}
		$this->addLocaleData();

		return true;
	}

	/**
	 * @return string[]
	 */
	public static function getDecisions(): array {
		$decisions = [
			SUBMISSION_EDITOR_DECISION_ACCEPT             => "Accept Submission",
			//1       Accept

			SUBMISSION_EDITOR_DECISION_PENDING_REVISIONS  => "Request Revisions",
			//2       Minor Revision

			SUBMISSION_EDITOR_DECISION_RESUBMIT           => "Resubmit for Review",
			// 3    Major Revision

			SUBMISSION_EDITOR_DECISION_DECLINE            => "Decline Submission",
			//4      Decline

			SUBMISSION_EDITOR_DECISION_SEND_TO_PRODUCTION => "Send To Production",
			//7      ignore

			SUBMISSION_EDITOR_DECISION_EXTERNAL_REVIEW    => "Send to Review",
			//8          first review round, ignore it

			SUBMISSION_EDITOR_DECISION_INITIAL_DECLINE    => "Decline Submission",
			//9      ignore as there is no review.

			SUBMISSION_EDITOR_RECOMMEND_ACCEPT            => "Recommendation: Accept Submission",
			//11

			SUBMISSION_EDITOR_RECOMMEND_PENDING_REVISIONS => "Recommendation: Request Revisions",
			//12

			SUBMISSION_EDITOR_RECOMMEND_RESUBMIT          => "Recommendation: Resubmit for Review",
			//13

			SUBMISSION_EDITOR_RECOMMEND_DECLINE           => "Recommendation: Decline Submission",
			//14
			SUBMISSION_EDITOR_DECISION_NEW_ROUND          => "New review round",
			//16           Major Revision
			SUBMISSION_EDITOR_DECISION_REVERT_DECLINE     => "Revert Decline",
			//17        ignore
		];

		return $decisions;
	}

	/**
	 * @copydoc Plugin::getName()
	 */
	function getName() {
		return 'ReviewerCreditsEditorialDecisionReportPlugin';
	}

	/**
	 * @copydoc Plugin::getDisplayName()
	 */
	function getDisplayName() {
		return __( 'plugins.generic.reviewerCredits.editorialDecisionReport.displayName' );
	}

	/**
	 * @copydoc Plugin::getDescription()
	 */
	function getDescription() {
		return __( 'plugins.generic.reviewerCredits.editorialDecisionReport.description' );
	}

	/**
	 * @copydoc ReportPlugin::display()
	 */
	function display( $args, $request ) {
		$context = $request->getContext();

		header( 'content-type: text/comma-separated-values' );
		header( 'content-disposition: attachment; filename=ReviewerCredits-editorialDecisions-' . date( 'Ymd' ) . '.csv' );
		AppLocale::requireComponents( LOCALE_COMPONENT_PKP_SUBMISSION );

		$editDecisionReportDao = DAORegistry::getDAO( 'ReviewerCreditsEditorialDecisionReportDAO' );
		$editorialDecisions    = $editDecisionReportDao->getEditorialDecisionReport( $context->getId() );

		import( 'classes.workflow.EditorDecisionActionsManager' );

		$decisions = self::getDecisions();

		$columns = [
			'journal_title' => "Journal Title",
			'journal_pissn' => "Journal p-ISSN",
			'journal_eissn' => "Journal e-ISSN",
			'submission_id' => "Manuscript ID",
			'round'         => "Round",
			'stage_id'      => "Stage Id",
			'decision'      => "Editorial Decision",
			'decision_date' => "Editorial Decision Date"
		];

		$editorDataEnabled = ( new ReviewerCreditsPlugin() )->getSetting( $context->getId(), 'reviewerCreditsEditorData' );
		if ( ! empty( $editorDataEnabled ) && $editorDataEnabled ) {
			$columns = array_merge( $columns,
				[
					'editor_name'    => "Editor Full Name",
					'editor_email'   => "Editor Email",
					'editor_orcid'   => "Editor Orcid",
					'editor_country' => "Editor Country"
				] );
		}

		$fp = fopen( 'php://output', 'wt' );
		//Add BOM (byte order mark) to fix UTF-8 in Excel
		fprintf( $fp, chr( 0xEF ) . chr( 0xBB ) . chr( 0xBF ) );
		fputcsv( $fp, array_values( $columns ) );

		$userDao = DAORegistry::getDAO( 'UserDAO' );

		$journal      = $request->getJournal();
		$journalTitle = $journal->getLocalizedPageHeaderTitle();
		$journalPIssn = $journal->getData( 'printIssn' );
		$journalEIssn = $journal->getData( 'onlineIssn' );

		foreach ( $editorialDecisions as $row ) {
//			error_log( "row: " . print_r( $row, true ) );
			$editor = $userDao->getById( $row->editor_id );

			foreach ( $columns as $index => $junk )
				switch ( $index ) {
					case 'journal_title':
						$columns[ $index ] = $journalTitle;
						break;
					case 'journal_pissn':
						$columns[ $index ] = $journalPIssn;
						break;
					case 'journal_eissn':
						$columns[ $index ] = $journalEIssn;
						break;
					case 'submission_id':
						$columns[ $index ] = 'S' . $row->submission_id . '-R' . $row->round;
						break;
					case 'round':
						$columns[ $index ] = $row->round;
						break;
					case 'stage_id':
						$columns[ $index ] = $row->stage_id;
						break;
					case 'decision_date':
						$columns[ $index ] = empty( $row->date_decided )
							? ""
							: DateTime::createFromFormat( 'Y-m-d H:i:s', $row->date_decided )
							          ->format( 'Y-m-d H:i:s' );
						break;
					case 'decision':
						$columns[ $index ] = isset( $decisions[ $row->$index ] ) ? $decisions[ $row->$index ] : $row->$index;
						break;

					//optional editor data
					case 'editor_name':
						$columns[ $index ] = $editor->getFullName();
						break;
					case 'editor_email':
						$columns[ $index ] = $editor->getEmail();
						break;
					case 'editor_orcid':
						$columns[ $index ] = $editor->getOrcid() ?? '';
						break;
					case 'editor_country':
						$columns[ $index ] = $editor->getCountryLocalized() ?? '';
						break;
					default:
						$columns[ $index ] = $row->$index;
				}
			fputcsv( $fp, $columns );
		}
		fclose( $fp );
	}

}
